var rename_timer=null;
var disable_contextmenu=false;
var filemanager_button;

function init_filemanager()
{
	register_filetype_handler(
		function(file) { return(file.dir) },
		function(file) { open_filemanager(file) }
	);

	var openicon_el=add_dock_icon(".desktop/Applications/Filemanager/filemanager.png",
	"File manager","Click to access your files.",1000,
	function() { open_filemanager(get_file("/")); });

	var trashcan_el=add_dock_icon(".desktop/Applications/Dock/trashcan.png",
	"Trashcan","Drop files here to delete them.",-1000);

	make_drop_target(trashcan_el,
		function(target,files,e) { return("Delete"); },
		function(target,files,e) { filemanager_delete_files(files); }
	);

	filemanager_init_context();
	filemanager_init_transfer();

	open_filemanager(get_file("/"));
}




function open_filemanager(directory)
{
	var window_el;
	if(window_el=find_filemanager_window(directory))
	{
		focus_window(window_el);
		return;
	}

	var size=get_registry("filemanager.size."+directory.fullpath);
	if(size) window_el=open_window(get_directory_name(directory),size[0],size[1],size[2],size[3]);
	else window_el=open_window(get_directory_name(directory));

	window_el.listener=directory.add_listener(function(directory) {
		if(directory.deleted) close_filemanager_window(window_el);
		else
		{
			filemanager_clear_selection(window_el);
			update_filemanager_contents(window_el);
			update_filemanager_status(window_el);
		}
	});

	make_drop_target(window_el.content,
		function(target,files,e) { return(filemanager_test_handler(target,files,e,directory)) },
		filemanager_drop_handler);

	window_el.tbody_el=make_element({ tag: "tbody" });
	window_el.table_el=make_element({ tag: "table", className: "filemanager", children: [
		{ tag: "col", width: "100%" },
		{ tag: "col", width: "0%" },
		{ tag: "col", width: "0%" },
		{ tag: "col", width: "0%" },
		window_el.tbody_el
	]});
	window_el.content.appendChild(window_el.table_el);

	window_el.close_handler=function(window_el) { close_filemanager_window(window_el) }

	window_el.resize_handler=window_el.move_handler=function(window_el) {
		store_filemanager_size(window_el);
	}

	window_el.table_el.onclick=function(e) {
		if(!e) e=window.event;
		if(!e.shiftKey&&!e.ctrlKey&&!e.metaKey) filemanager_clear_selection(window_el);
	}

	var context_el=is_safari()?window_el.table_el:window_el.content;
	add_context_handler(context_el,function(x,y) {
		filemanager_context_handler(window_el,x,y)
	});

	document.oncontextmenu=function(e) { return(false); }
	window_el.id="filemanager_"+directory.toString();
	window_el.directory=directory;
	window_el.selected=[];

	refresh_filemanager(window_el);
}

function close_filemanager_window(window_el)
{
	window_el.directory.remove_listener(window_el.listener);
	close_window(window_el);
}

function find_filemanager_window(directory)
{
	return(document.getElementById("filemanager_"+directory.toString()));
}

function refresh_filemanager(window_el)
{
	set_window_status(window_el,"Reading directory contents...");

	window_el.directory.read_directory(function(directory,error) {
		if(error)
		{
			set_window_status(window_el,"Couldn't read file list: "+error);
		}
		else
		{
			window_el.directory.sort_files("alpha");
			update_filemanager_contents(window_el);
		}
	});
}

function store_filemanager_size(window_el)
{
	set_registry("filemanager.size."+window_el.directory.fullpath,
	get_window_position(window_el).concat(get_window_size(window_el)));
}


// content handling

function update_filemanager_contents(window_el)
{
	if(!window_el) return;

	var list=window_el.directory.files;
	var tbody=window_el.tbody_el;

	filemanager_clear_selection(window_el);

	while(tbody.firstChild) tbody.removeChild(tbody.firstChild);

	var tr_el=make_element({ tag: "tr", noselect: true, children: [
		{ tag: "th", className: "filename", width: "100%", children: ["Filename"] },
		{ tag: "th", className: "size", width: "0%", children: ["Size"] },
		{ tag: "th", className: "date", width: "0%", children: ["Date"] },
		{ tag: "th", className: "permissions", width: "0%", children: ["Permissions"] }
	]});
	tbody.appendChild(tr_el);

	window_el.file_elements=[];

	for(var i=0;i<list.length;i++)
	{
		var size=list[i].dir?"":space_number(list[i].size);
		var datestr=display_date(list[i].modify);
		var modes=list[i].permissions;
		if(list[i].owner||list[i].group) modes+=" ("+list[i].owner+":"+list[i].group+")";

		var namecell_el=make_element({ tag: "td", className: "filename", children: [
			{ tag: "img", src: list[i].icon_large, noselect: true },
			{ tag: "span", noselect: true, children: [list[i].filename] }
		]})
		namecell_el.file=list[i];

		var tr_el=make_element({ tag: "tr", children: [
			namecell_el,
			{ tag: "td", className: "size", noselect: true, children: [ size ] },
			{ tag: "td", className: "date", noselect: true, children: [ datestr ] },
			{ tag: "td", className: "permissions", noselect: true, children: [ modes ] },
		]});

		tbody.appendChild(tr_el);

		namecell_el.onmousedown=function(namecell_el)
		{ return function(e) { filemanager_click_handler(e,window_el,namecell_el); } }(namecell_el);

		namecell_el.onclick=function(e) { if(!e) e=window.event; e.cancelBubble=true; }

		namecell_el.ondblclick=function(e) { filemanager_doubleclick_handler(e,window_el); }

		if(list[i].dir) make_drop_target(namecell_el,filemanager_dir_test_handler,filemanager_dir_drop_handler);

		window_el.file_elements.push(namecell_el);
	}

	update_filemanager_status(window_el);
}

function update_filemanager_status(window_el)
{
	var files=window_el.directory.files.length;
	var selected=window_el.selected.length;
	var total_size=0,selected_size=0;
	var status;

	for(var i=0;i<window_el.directory.files.length;i++) total_size+=window_el.directory.files[i].size;
	for(var i=0;i<window_el.selected.length;i++) selected_size+=window_el.selected[i].file.size;

	status=pluralize(files,"item")+", "+display_size(total_size);

	if(selected) status=pluralize(selected,"item")+", "+display_size(selected_size)+" selected (out of "+status+")";

	set_window_status(window_el,status);
}



// icon handlers

var filemanager_click_x,filemanager_click_y;

function filemanager_click_handler(e,window_el,namecell_el)
{
	if(!e) e=window.event;

	close_menu();

	var unselect_files=[];
	var rename_click=false;

	var threshold=get_registry("filemanager_drag_threshold",8);

	filemanager_click_x=get_mouse_x(e);
	filemanager_click_y=get_mouse_y(e);
	filemanager_button=e.button;

	if(e.shiftKey)
	{
		var old_index=list_index(window_el.file_elements,window_el.select_element||window_el.file_elements[0]);
		var new_index=list_index(window_el.file_elements,namecell_el);

		if(old_index>new_index) { var tmp=old_index; old_index=new_index; new_index=tmp; }

		var select_func;
		if(window_el.select_element&&!filemanager_file_selected(window_el,window_el.select_element)) select_func=filemanager_unselect_file;
		else select_func=filemanager_select_file;

		if(!e.ctrlKey&&!e.metaKey) filemanager_clear_selection(window_el);
		for(var i=old_index;i<=new_index;i++) select_func(window_el,window_el.file_elements[i]);
	}
	else if(e.ctrlKey||e.metaKey)
	{
		if(filemanager_file_selected(window_el,namecell_el))
		{
			unselect_files.push(namecell_el);
		}
		else filemanager_select_file(window_el,namecell_el);

		window_el.select_element=namecell_el;
	}
	else
	{
		if(filemanager_file_selected(window_el,namecell_el))
		{
			if(e.button!=2)
			{
				for(var i=0;i<window_el.selected.length;i++)
				if(window_el.selected[i]!=namecell_el) unselect_files.push(window_el.selected[i]);

				rename_click=true;
			}
		}
		else
		{
			filemanager_clear_selection(window_el);
			filemanager_select_file(window_el,namecell_el);
		}

		window_el.select_element=namecell_el;
	}

	if(e.button!=2) start_hold_timer(namecell_el,e,function(x,y) {
		document.onmouseup=null;
		document.onmousemove=null;
		disable_contextmenu=false;
		filemanager_context_handler(window_el,x,y);
	});
	else disable_contextmenu=true;

	document.onmousemove=function(e)
	{
		if(!e) e=window.event;

		var dx=get_mouse_x(e)-filemanager_click_x;
		var dy=get_mouse_y(e)-filemanager_click_y;

		if(window_el.selected.length)
		if(dx<-threshold||dy<-threshold||dx>threshold||dy>threshold)
		{
			var files=[];
			for(var i=0;i<window_el.selected.length;i++) files.push(window_el.selected[i].file);

			stop_hold_timer(namecell_el);
			start_drag(e,files);
		}

		test_hold_timer(namecell_el,e);
	}

	document.onmouseup=function(e)
	{
		if(!e) e=window.event;

		document.onmouseup=null;
		document.onmousemove=null;

		if(e.button==2)
		{
			disable_contextmenu=false;
			filemanager_context_handler(window_el,get_mouse_x(e),get_mouse_y(e));
		}
		else
		{
			for(var i=0;i<unselect_files.length;i++)  filemanager_unselect_file(window_el,unselect_files[i]);

			if(rename_click&&!rename_timer) rename_timer=setTimeout(function() {
				rename_timer=null;
				filemanager_rename_file(window_el,namecell_el);
			},get_registry("filemanager_rename_delay",250));

			focus_window(window_el);
		}

		stop_hold_timer(namecell_el,e);
	}

	cancel_drag(e);
}

function filemanager_doubleclick_handler(e,window_el)
{
	if(rename_timer)
	{
		clearTimeout(rename_timer);
		rename_timer=null;
	}
	for(var i=0;i<window_el.selected.length;i++) handle_file(window_el.selected[i].file);
}



function filemanager_rename_file(window_el,icon_el)
{
	var name_el=icon_el.childNodes[1];

	icon_el.removeChild(name_el);
	icon_el.onmousedown=icon_el.ondblclick=null;
	//name_el.style.display="none";

	var edit_el=make_element({ tag: "input", type: "text", value: icon_el.file.filename });

	var apply_name=function() {
		var new_name=edit_el.value;

		if(new_name==icon_el.file.filename)
		{
			update_filemanager_contents(window_el);
			return;
		}

		edit_el.setAttribute("disabled","true");
		set_window_status(window_el,"Renaming file...");

		icon_el.file.rename(new_name,function(newfile,error) {
			if(error)
			{
				update_filemanager_contents(window_el);
				set_window_status(window_el,"Couldn't rename the file: "+error);
			}
		});
	};

	edit_el.onblur=function(e) { apply_name(); }

	edit_el.onkeypress=function(e) {
		if(!e) e=window.event;
		if(e.keyCode==13) edit_el.blur();
		else if(e.keyCode==27)
		{
			edit_el.onblur=null;
			edit_el.blur();
			update_filemanager_contents(window_el);
		}
	}

	icon_el.appendChild(edit_el);
	edit_el.focus();
}

function filemanager_delete_files(files)
{
	if(confirm("Are you sure you want to delete "+name_files(files)+"?"))
	{
		delete_files(files,function(failure,errors) {
			if(failure.length) alert("The following files could not be deleted:\n"+failure.join("\n"));
		});
	}
}



register_application(init_filemanager);
